import os
from pprint import pformat
import logging

from flask import Flask, render_template, abort, jsonify, session, url_for, request, redirect

from p2.ui.sessions import settings
from p2.store.client import Client
from p2.logging import HttpLogger

app = Flask(__name__)
app.config.from_object('p2.ui.sessions.settings')
app.secret_key = app.config['SECRET_KEY']

log = HttpLogger(
    level = logging.DEBUG,
    silent = False,
    name='p2.ui.resources'
)
logger = log.getLogger()


p2_store_url = os.getenv('P2_STORE_URL')
if p2_store_url is None:
    raise Exception("No P2_STORE_URL envar found")

client = Client(p2_store_url)

@app.route('/version')
def version():
    return "p2.ui.settings app version 0.1.0"

@app.route('/menu')
def menu():
    return jsonify(settings.MENU)

@app.route('/api/display_sessions_list')
def display_sessions_list():
    where=dict()
    sort=None
    if 'month' in request.args:
        sort='dom'
    for key in ['dom', 'month']:
        if request.args.get(key):
            where[key] = "%02d" % int(request.args[key])
    if request.args.get('year'):
        where['year'] = request.args['year']

    sessions = client.select(
        table='session_report_view',
        filter=where,
        sort=sort,
        fetch=[
            'exp_id',
            'filepath',
            'mins',
            'hour',
            'dom',
            'month',
            'year',
            'start_time',
            'end_time',
            'context',
            'status',
            'load_injectors',
            'test_result'
        ])
    if sessions is None:
        raise Exception("Error: no sessions found")
    return jsonify(sessions)

@app.route('/api/display_session_log')
def display_session_log():
    exp_id = request.args.get('exp_id')
    log_text = client.select(
        table='exp_logs',
        filter={
            'exp_id': exp_id,
            'log_type': 1
        },
        fetch=['log_text']
    )
    if log_text is None or len(log_text) != 1:
        abort(404)
    else:
        return jsonify(log_text[0]['log_text'])

@app.route('/api/display_session_exp')
def display_session_exp():
    exp_id = request.args.get('exp_id')
    filepath = request.args.get('filepath')
    status = request.args.get('status')

    app.logger.info(f"exp_id  : {exp_id}")
    app.logger.info(f"filepath: {filepath}")
    app.logger.info(f"status  : {status}")

    data = dict()

    exp = client.select(
        table='experiments',
        filter={
            'filename': filepath
        },
        fetch=['data', 'type']
    )

    if exp is None or len(exp) != 1:
        abort(404)
    else:
        data[exp[0]['type'].capitalize()] = pformat(
            exp[0]['data'], indent=2, width=80
        )

    if status != 'scheduled':
        exp = client.select(
            table='experiments',
            filter={
                'filename': exp_id
            },
            fetch=['data', 'type']
        )
        if exp is not None and len(exp) == 1:
            data[exp[0]['type'].capitalize()] = pformat(
                exp[0]['data'], indent=2, width=80
            )

    return jsonify(data)

@app.route('/api/update_session_status/<exp_id>')
def update_session_status(exp_id):
    status = request.args.get('status')
    logger.info(f"Going to update session {exp_id} status to {status} ...")
    ret = client.update(
        table='exp_session',
        where={'exp_id': exp_id},
        record={'status': status}
    )
    if ret != 1:
        abort(404)
    else:
        return exp_id
