var display_sessions_url="/sessions/api/display_sessions_list"
var display_session_log="/sessions/api/display_session_log"
var display_session_exp="/sessions/api/display_session_exp"

function refresh_data( time_frame ) {
    var data  = [];
    $.getJSON( display_sessions_url, "time_frame="+time_frame, function( d ) {
        data = d;
    });
    return data;
};

var sessions_source = {
    datatype: "json",
    datafields: [
        { name: 'exp_id' },
        { name: 'filepath' },
        { name: 'mins' },
        { name: 'hour' },
        { name: 'dom' },
        { name: 'month' },
        { name: 'year' },
        { name: 'start_time', type: 'date', format: 'yyyy-MM-ddTHH:mm:ss.ffffff' },
        { name: 'end_time', type: 'date', format: 'yyyy-MM-ddTHH:mm:ss.ffffff' },
        { name: 'context' },
        { name: 'status' },
        { name: 'load_injectors' },
        { name: 'test_result' }
    ],
    sortcolumn: 'start_time',
    sortdirection: 'desc',
    url: display_sessions_url
};
var sessions_dataAdapter = new $.jqx.dataAdapter(sessions_source);

var time_frame;

function refresh_session_grid() {
    var tmpS = $("#sessions_grid").jqxGrid('source');

    var url = display_sessions_url
    var today = new Date();
    var dom = today.getDate();
    var month = today.getMonth()+1
    var year = today.getFullYear()

    if (time_frame == "today") {
        url = url + "?dom="+dom+"&month="+month+"&year="+year;
    } else if (time_frame == "current_month") {
        url = url + "?month="+month+"&year="+year;
    } else if (time_frame == "next_month") {
        if (month < 12) {
            url = url + "?month="+(month+1)+"&year="+year;
        } else {
            url = url + "?month=1"+"&year="+year+1;
        }
    } else if (time_frame == "previous_month") {
        if (month > 1) {
            url = url + "?month="+(month-1)+"&year="+year;
        } else {
            url = url + "?month=12"+"&year="+year-1;
        }
    }
    tmpS._source.url = url
    console.log("tmpS._source.url: ", tmpS._source.url)
    $('#sessions_grid').jqxGrid('source', tmpS);
};

function display_sessions_list() {

    $("#appWorkspace").append('<div id="sessions_menubar" class="p2-menu" bp=12></div>');
    $("#appWorkspace").append('<div id="sessions_grid" bp=12></div>');

    var source = [{label: "All", selected: true},
                  {label: "Today",          value: "today"},
                  {label: "Previous Month", value: "previous_month"},
                  {label: "Current Month",  value: "current_month"},
                  {label: "Next Month",     value: "next_month"},
                  //{label: "Add Session",    value: "add_session"}
    ];
    $('#sessions_menubar').jqxMenu({source: source, width:'100%', height: '30px'});
    $('#sessions_menubar').on('itemclick', function(event) {
        if ($('#session_window').length) {
            $('#session_window').remove();
        }
        time_frame = $(event.target).attr('item-value');
        console.log("sessions of ", time_frame);
        refresh_session_grid();
    });

    var cellclass = function (row, columnfield, value) {
        if (value == 'completed') {
            return 'green_ok';
        }
        else if (value == 'failed') {
            return 'red_not_ok';
        }
        else if (value == 'deleted') {
            return 'no_value_or_deleted';
        }
    }

    var boolean_class = function (row, columnfield, value) {
        if ($.type(value) == 'boolean' && value == false) {
            return 'red_not_ok';
        } 
        else if (value == true) {
            return 'green_ok';
        }
        else {
            return 'no_value_or_deleted'
        }
    }

    var load_injectors_renderer = function (row, columnfield, value, defaulthtml, columnproperties) {
        if ($.type(value) == 'boolean' && value == true) {
            return '<div class="jqx-grid-cell-center-align" style="margin-left: 2px;margin-top: 8px">Load</div>';
        } else if ($.type(value) == 'boolean' && value == false) {
            return '<div class="jqx-grid-cell-center-align" style="margin-left: 2px;margin-top: 8px">No load</div>';
        } else {
            return '<span> </span>';
        }
    };

    var test_result_renderer = function (row, columnfield, value, defaulthtml, columnproperties) {
        if ($.type(value) == 'boolean' && value == true) {
            return '<div class="jqx-grid-cell-center-align" style="margin-left: 2px;margin-top: 8px">Passed</div>';
        } else if ($.type(value) == 'boolean' && value == false) {
            return '<div class="jqx-grid-cell-center-align" style="margin-left: 2px;margin-top: 8px">Failed</div>';
        } else {
            return '<span> </span>';
        }
    };

    $("#sessions_grid").jqxGrid({
        width: '98%',
        autoheight: true,
        sortable: true,
        pageable: true,
        selectionmode: 'singlerow',
        enablebrowserselection: true,
        pagesizeoptions: ['10', '20', '30'],
        pagesize: 20,
        source: sessions_dataAdapter,
        columns: [
            { text: 'Exp. Session Id', datafield: 'exp_id', width: "25%" },
            { text: 'Context', datafield: 'context', width: "20%" },
            { text: 'Min', datafield: 'mins', width: "3%", sortable: false },
            { text: 'Hour', datafield: 'hour', width: "3%" },
            { text: 'Day', datafield: 'dom', width: "3%" },
            { text: 'Month', datafield: 'month', width: "3%" },
            { text: 'Year', datafield: 'year', width: "5%" },
            { text: 'Start Date', datafield: 'start_time', cellsformat: 'yyyy-MM-dd HH:mm', width: "10%" },
            { text: 'End Date', datafield: 'end_time', cellsformat: 'yyyy-MM-dd HH:mm', width: "10%" },
            { text: 'Status', datafield: 'status', cellclassname: cellclass, width: "10%" },
            { text: 'Load', datafield: 'load_injectors', width: "4%", cellsalign: 'center', cellsrenderer: load_injectors_renderer, cellclassname: boolean_class },
            { text: 'Pass/Fail', datafield: 'test_result', width: "4%", cellsalign: 'center', cellsrenderer: test_result_renderer, cellclassname: boolean_class }
        ]
    });

    $('#sessions_grid').on('rowselect', function (event) {
        if ($('#session_window').length) {
            $('#session_window').remove();
        }
        var args = event.args;
        var row_index = args.rowindex;
        var row = args.row;
        console.log("Exp Id: "+row.exp_id);
        console.log("Filepath: "+row.filepath);
        display_session_details(row_index, row);
    });
};

function session_report_updated() {
    $('#sessions_grid').jqxGrid('refreshdata');
    $('#sessions_grid').jqxGrid('refresh');
    refresh_session_grid();
    display_ack_window("", "Session Report updated");
}

function fields_to_boolean (record) {
    if ($.type(record.load_injectors) == 'boolean' && $.type(record.test_result) == 'boolean') {
        return;
    }

    if ( record.load_injectors == 'Load') {
        record.load_injectors = true;
    } else {
        record.load_injectors = false;
    };

    if (record.test_result == 'Pass') {
        record.test_result = true;
    } else {
        record.test_result = false;
    };
}

function fields_to_string (record) {
    if (record == null) {
        return;
    }
    if ($.type(record.load_injectors) != 'boolean' && $.type(record.test_result) != 'boolean') {
        return;
    }
    if ( record.load_injectors == true) {
        record.load_injectors = 'Load';
    } else {
        record.load_injectors = 'No Load';
    }
    if (record.test_result == true) {
        record.test_result = 'Pass';
    } else {
        record.test_result = 'Fail';
    }
}

// the session report...
var report_template = {
    load_injectors: 'Load',
    test_result: 'Pass',
    notes: ""
};
var report = Object.assign({}, report_template);

function display_session_report( exp_id ) {
    var template = [{
        bind: 'load_injectors',
        type: 'option',
        label: 'Load Injectors',
        required: true,
        labelWidth: '140px',
        width: '100px',
        component: 'jqxDropDownList',
        options: [
            { value: 'Load' },
            { value: 'No Load'}
        ]
    }, {
        bind: 'test_result',
        type: 'option',
        label: 'Test Result',
        required: true,
        labelWidth: '140px',
        width: '100px',
        component: 'jqxDropDownList',
        options: [
            { value: 'Pass' },
            { value: 'Fail'}
        ]
    },{
        type: 'blank',
        rowHeight: '10px'
    },{
        required: false,
        bind: 'notes',
        name: 'notesTextArea',
        type: 'custom',
        label: 'Notes',
        labelWidth: '140px',
        width: '200px',
        component: 'jqxTextArea',
        init: function(component) {
            component.jqxTextArea({
                width: "90%",
                height: 100
            });
        } 
    },{
        type: 'blank',
        rowHeight: '10px'
    },{
        columns: [{
            type: 'button',
            name: 'saveButton',
            rowHeight: '40px',
            columnWidth: '120px',
            init: function(component) {
                component.jqxButton({
                    theme: 'darkblue',
                    value: 'Save',
                    width: '100px',
                    height: '25px',
                    textPosition: 'center'
                });
            }
        }]
    }];

    $('#session_report_form').jqxForm({
        template: template,
        value: report,
        padding: { left: 10, top: 10, right: 10, bottom: 10 }
    });

    p2store_select(
        table='session_report',
        params={
            filter:{exp_id: exp_id},
            sort:'exp_id asc',
            limit:1
        },
        callback=load_session_report
    );

    $('#session_report_form').on('formDataChange', function (event) {
        var args = event.args;
        report = args.value;
        fields_to_boolean(report);
        console.log(report);
    });

    $('#session_report_form').on('buttonClick', function (event) {
        var args = event.args;
        var name = args.name;

        if (name == 'saveButton') {
            nta = $('#session_report_form').jqxForm('getComponentByName', 'notesTextArea');
            report.notes = nta.val();
            console.log("Saving record for exp_id:" + exp_id);
            fields_to_boolean(report);
            report['exp_id'] = exp_id;
            console.log(report);
            p2store_update_on_conflict(
                table='session_report',
                conflicting_cols=['exp_id'],
                record=report,
                callback=session_report_updated
            );
        }
    });
};

function load_session_report(records) {
    if (records.length == 1) {
        report = records[0];
    } else {
        report = report_template;
    }
    fields_to_string(report);
    $('#session_report_form').val(report);
}

function display_session_details(row_index, row) {
    $.getJSON( display_session_exp, {exp_id: row.exp_id, filepath: row.filepath, status: row.status}, function( d ) {
        console.log(d)

        if (Object.keys(d).length > 0) {

            $("#appWorkspace").append('<div id="session_window"><div>Session '+row.exp_id+' details</div></div>');

            var ribbon_elem='<div><div id="selected_session_ribbon" ><ul>';
            for (var key in d) {
                ribbon_elem += '<li>'+key+'</li>';
            }
            // adding a session report ribbon if status is not scheduled...
            if (row.status != 'scheduled') {
                ribbon_elem += '<li>Session Report</li>';
            }
            ribbon_elem += '</ul>';
            ribbon_elem += '<div>';
            for (var key in d) {
                ribbon_elem += '<div>'+d[key]+'</div>';
            }
            // adding a session report ribbon if status is not scheduled...
            if (row.status != 'scheduled') {
                ribbon_elem += '<div id="session_report_form"></div>';
            }

            ribbon_elem += '</div>';
            ribbon_elem += '</div>';
            if (row.status == 'scheduled') {
                ribbon_elem += '<input id="status_session_button" type="button" value="Delete Session" />';
            }
            if (row.status == 'deleted') {
                ribbon_elem += '<input id="status_session_button" type="button" value="Schedule Session" />';
            }
            ribbon_elem += '</div>';
            $("#session_window").append(ribbon_elem);
            $("#selected_session_ribbon").jqxRibbon({width: "100%", height: "620px", mode: "default"});
            if (row.status == 'scheduled') {
                $("#status_session_button").jqxButton({ theme: 'orange', width: '150', height: '25'});
                $("#status_session_button").on('click', function () {
                    display_session_ack_window(row_index, row, 'delete', 'deleted');
                });
            }
            if (row.status == 'deleted') {
                $("#status_session_button").jqxButton({ theme: 'energyBlue', width: '150', height: '25'});
                $("#status_session_button").on('click', function () {
                    display_session_ack_window(row_index, row, 'schedule', 'scheduled');
                });
            }
            $('#session_window').jqxWindow({
                width: "800px",
                height: "700px",
                position:'top, right',
                resizable: true,
                isModal: true,
                closeButtonAction:'close'
            });
            $('#selected_session_ribbon').jqxRibbon('render');

            display_session_report(row.exp_id);
            display_execution_log(row.exp_id);

            $('#session_window').on('close', function (event) {
                $('#sessions_grid').jqxGrid('clearselection');
                $('#session_window').remove();
            });
        }
    });
};

function display_execution_log(exp_id) {
    $.getJSON( display_session_log, "exp_id="+exp_id, function( d ) {
        text = d;
        if (d !== undefined && d !== null) {
            console.log("Text:"+text)
            $('#selected_session_ribbon').jqxRibbon('addAt', 0, {title: 'Execution Log', content: text});
            $('#selected_session_ribbon').jqxRibbon('render');
        }
    });
};

function display_session_ack_window(row_index, row, verb, new_status) {

    let title = "Are you really sure you want to "+verb+" test session "+row.exp_id+" ?";

    session_ack_window = `
        <div id="ackDialog" style="display:none">
            <div id="session_ack_window">
                <div>You are going to `+verb+` a test session...</div>
                <div>
                    <div id="session_window_title">`+title+`</div>
                    <input type="button" value="Cancel" id="cancel_session_ack_button" />
                    <input type="button" value="`+capitalize(verb)+`" id="okay_session_ack_button" />
                </div>
            </div>
        </div>
    `;
    $("#appWorkspace").append(session_ack_window);

    $("#session_ack_window").jqxWindow({ 
        height: 150,
        width: 'auto', 
        autoOpen: false, 
        isModal: true, 
        okButton: $('#okay_session_ack_button'), 
        cancelButton: $('#cancel_session_ack_button'),
        initContent: function () {
            $("#okay_session_ack_button").jqxButton({ theme: 'orange', width: 100, height: 30 });
            $("#cancel_session_ack_button").jqxButton({ width: 100, height: 30 });
            $('#cancel_session_ack_button').focus();
            
            $("#okay_session_ack_button").click(function () {
                $.ajax({
                    url: '/sessions/api/update_session_status/'+row.exp_id,
                    data: {'status': new_status},
                    success: function(result) {
                        row.status = new_status;
                        $("#sessions_grid").jqxGrid('updaterow', row_index, row);
                        $("#session_window").jqxWindow('close');
                    },
                    error: function(request,msg,error) {
                        alert(capitalize(verb)+" action error on session "+row.exp_id);
                    }
                });
            });
        }
    });

    $('#session_ack_window').on('close', function (event) {
        $("#ackDialog").remove();
        $("#session_ack_window").remove();
    });

    $("#session_ack_window").jqxWindow('open');
    $("#session_ack_window").jqxWindow('focus');
};

function capitalize(s)
{
    return s[0].toUpperCase() + s.slice(1);
}
